@@ -1,9 +1,9 @@
-cvs_id = "$Id: Event.py,v 1.3 2004/04/22 18:41:16 jpakaste Exp $"
-
 import traceback
 import sys
 
-def find_bases(klass, d = {}):
+def find_bases(klass, d = None):
+    if d is None:
+        d = {}
     for b in klass.__bases__:
         if issubclass(b, BaseSignal):
             d[b] = 1
@@ -44,9 +44,7 @@ class SignalEmitter:
     def emit_signal(self, signal):
         """Emit a signal object. Signal should be an instance of BaseSignal or
         a subclass."""
-        # huh, what? suddenly if i don't specify second argument, the resuls
-        # are all weird
-        sclasses = find_bases(signal.__class__, {})
+        sclasses = find_bases(signal.__class__)
         sclasses.append(signal.__class__)
         for sc in sclasses:
             for handler, data in self._slots.get(sc, {}).items():
@@ -56,7 +54,7 @@ class SignalEmitter:
                     args = (signal,)
                 try:
                     apply(handler, args)
-                except Exception, ex:
+                except:
                     sys.stderr.write("Caught an exception when trying to "
                                      "call a signal handler for "
                                      "signal %s\n" %
@@ -70,13 +68,44 @@ class ConfigOptionSignal(BaseSignal):
 class LocalConfigOptionSignal(ConfigOptionSignal):
     pass
 
+class FinderDoneSignal(BaseSignal):
+    """
+    Used by: SubscribeDialog.py
+
+    Emitted by FeedFinder when it's finished looking for feeds
+    """
+    def __init__(self, sender, feeds, error=None):
+        BaseSignal.__init__(self,sender)
+        self.feeds = feeds
+        self.error = error
+
+class AuthNeededSignal(BaseSignal):
+    """
+    Used by: SubscribeDialog.py
+
+    Emitted when feed URL needs authorization
+    """
+    def __init__(self, sender):
+        BaseSignal.__init__(self, sender)
+
+class FeedSummaryDoneSignal(BaseSignal):
+    """
+    Used by: SubscribeDialog.py
+
+    Emitted by FeedSummaryPoller after the summary has been parsed.
+    """
+    def __init__(self, sender, feed, data, error=None):
+        BaseSignal.__init__(self, sender)
+        self.feed = (feed, data)
+        self.error = error
+
 class PollChangedSignal(BaseSignal):
     def __init__(self, sender):
         BaseSignal.__init__(self, sender)
 
 class FeedsChangedSignal(LocalConfigOptionSignal):
     def __init__(self, sender):
-        BaseSignal.__init__(self, sender)
+        LocalConfigOptionSignal.__init__(self, sender)
 
 class FeedDeletedSignal(FeedsChangedSignal):
     def __init__(self, sender, feed):
@@ -139,8 +168,24 @@ class FeedErrorStatusChangedSignal(BaseSignal):
     pass
 
 class FeedCategoriesChangedSignal(BaseSignal):
+    pass
+
+class FeedCategoryListChangedSignal(FeedCategoriesChangedSignal):
+    pass
+
+class FeedCategoryAddedSignal(FeedCategoryListChangedSignal):
+    def __init__(self, sender, category):
+        FeedCategoryListChangedSignal.__init__(self, sender)
+        self.category = category
+
+class FeedCategoryRemovedSignal(FeedCategoryListChangedSignal):
+    def __init__(self, sender, category):
+        FeedCategoryListChangedSignal.__init__(self, sender)
+        self.category = category
+
+class FeedCategoryChangedSignal(FeedCategoriesChangedSignal):
     def __init__(self, sender, feed=None):
-        BaseSignal.__init__(self, sender)
+        FeedCategoriesChangedSignal.__init__(self, sender)
         self.feed = feed
 
 class ItemsAddedSignal(BaseSignal):
@@ -183,5 +228,11 @@ class ImageUpdatedSignal(BaseSignal):
         self.url = url
         self.data = data
 
+class SubscriptionContentsUpdatedSignal(BaseSignal):
+    pass
+
+class PollingStoppedSignal(BaseSignal):
+    pass
+
 if __name__ == '__main__':
     print find_bases(ItemReadSignal)
