@@ -1,5 +1,6 @@
-import gtk, straw, gobject
 from error import log, logparam, logtb
+from xml.sax import saxutils
+import gtk, straw, gobject, gnome
 import time
 
 class FeedPropertiesDialog:
@@ -27,6 +28,14 @@ class FeedPropertiesDialog:
         self._previous_refresh_label = xml.get_widget(
             'properties_previous_refresh_label')
         self._categories_treeview = xml.get_widget('feed_categories_treeview')
+        self._feed_info_href_container = xml.get_widget(
+            'feed_info_href_container')
+        self._feed_info_description = xml.get_widget('feed_info_description')
+        self._feed_info_copyright = xml.get_widget('feed_info_copyright')
+
+        self._feed_info_href = gnome.ui.HRef(url='', text='')
+        self._feed_info_href_container.pack_start(
+            self._feed_info_href, expand=gtk.FALSE, fill=gtk.FALSE)
 
         self._title_default = ''
         self._url_default = ''
@@ -84,11 +93,11 @@ class FeedPropertiesDialog:
     def member_toggled(self, cell, path):
         model = self._categories_treeview.get_model()
         category = model[path][self.COLUMN_OBJECT]
-        if self._feed in category:
-            category.remove(self._feed)
+        if self._feed in category.feeds:
+            category.remove_feed(self._feed)
             model[path][self.COLUMN_MEMBER] = False
         else:
-            category.append(self._feed)
+            category.append_feed(self._feed, False)
             model[path][self.COLUMN_MEMBER] = True
 
     def display_properties(self, feed):
@@ -136,10 +145,10 @@ class FeedPropertiesDialog:
             iter = model.append()
             model.set(iter,
                       self.COLUMN_NAME, c.title,
-                      self.COLUMN_MEMBER, feed in c,
+                      self.COLUMN_MEMBER, feed in c.feeds,
                       self.COLUMN_OBJECT, c)
 
-        format = '%A, %d %B %Y %H:%M'
+        format = straw.utils.get_date_format()
         self._previous_refresh_label.set_text(
             time.strftime(format, time.localtime(feed.last_poll)))
         next = feed.next_refresh
@@ -150,8 +159,44 @@ class FeedPropertiesDialog:
         else:
             self._next_refresh_label.hide()
 
+        self._display_feed_information(feed)
         self._restore_button.set_sensitive(gtk.FALSE)
 
+    def _display_feed_information(self, feed):
+        title = straw.utils.convert_entities(feed.channel_title)
+        if len(title) == 0:
+            title = feed.title
+        if len(title) == 0:
+            title = feed.channel_link
+        title = title.strip()
+
+        link = feed.channel_link.strip()
+
+        if len(title) > 0:
+            self._feed_info_href.set_text(saxutils.escape(title))
+            self._feed_info_href.set_url(link)
+            self._feed_info_href.show()
+            self._feed_info_href.set_sensitive(len(link) > 0)
+        else:
+            self._feed_info_href.hide()
+
+        description = feed.channel_description.strip()
+        size = len(description)
+        if size and description != title:
+            if straw.utils.is_html(description):
+                description = straw.utils.read_text(description, size)
+            self._feed_info_description.set_text(straw.utils.convert_entities(description))
+            self._feed_info_description.show()
+        else:
+            self._feed_info_description.hide()
+
+        copyright = feed.channel_copyright
+        self._feed_info_copyright.set_text(copyright)
+        if len(copyright):
+            self._feed_info_copyright.show()
+        else:
+            self._feed_info_copyright.hide()
+
     def restore_defaults(self):
         # FIXME: add frequency and number of items and the default flags
         self._feed.title = self._title_default
