@@ -1,5 +1,3 @@
-cvs_id = "$Id: ItemStore.py,v 1.3 2004/04/22 18:41:16 jpakaste Exp $"
-
 import cPickle as pickle
 import os, sys
 import time
@@ -481,6 +479,7 @@ def stringify_item(item):
         'fm_license': item.fm_license,
         'fm_changes': item.fm_changes,
         'creator': item.creator,
+        'contributors': item.contributors,
         'license_urls': item.license_urls,
         'publication_name': item.publication_name,
         'publication_volume': item.publication_volume,
@@ -513,6 +512,7 @@ def unstringify_item(itemstring):
     item.fm_license = dict.get('fm_license', None)
     item.fm_changes = dict.get('fm_changes', None)
     item.creator = dict.get('creator', None)
+    item.contributors = dict.get('contributors', None)
     item.license_urls = dict.get('license_urls', None)
     item._sticky = dict.get('sticky', 0)
     item.publication_name = dict.get('publication_name', None)
