@@ -1,5 +1,3 @@
-cvs_id = "$Id: MainWindow.py,v 1.5 2004/04/24 12:01:23 jpakaste Exp $"
-
 import StringIO
 
 import gobject
@@ -8,13 +6,16 @@ import gnome
 import gtkhtml2
 import gnome.ui
 import pango
+import time
+import os
+import straw
+import utils
+import urlparse
+import re
+import SubscribeDialog
+from error import log, logparam, logtb
 from gtk import glade
 from xml.sax import saxutils
-import time
-
-import straw
-from error import log, logparam, logtb
-
 
 class DummyDocStream(StringIO.StringIO):
     def write_stream(self, s):
@@ -23,7 +24,6 @@ class DummyDocStream(StringIO.StringIO):
 class ItemView:
     def __init__(self, item_view_container):
         self._item = None
-        self._date_format = '%A, %d %B %Y %H:%M'
         self._encoding = straw.utils.get_locale_encoding()
         self._mmgr = straw.MessageManager.get_instance()
 
@@ -66,6 +66,7 @@ class ItemView:
         try:
             gnome.url_show(link)
         except Exception, ex:
+            print ex
             straw.hig_alert.reportError(_("Error Loading Browser"),
                                         _("Straw cannot find a browser to view this item. Please check your browser settings and try again."))
         return
@@ -74,8 +75,14 @@ class ItemView:
         try:
             feed = straw.main.get_visible_feed()
             url = straw.utils.complete_url(url, feed.location)
-            image = straw.ImageCache.cache[url]
-            stream.write(image.get_data())
+            if urlparse.urlparse(url)[0] == 'file':
+                # local URL, don't use the cache.
+                f = file(urlparse.urlparse(url)[2])
+                stream.write(f.read())
+                f.close()
+            else:
+                image = straw.ImageCache.cache[url]
+                stream.write(image.get_data())
         except Exception, ex:
             log("Error reading image in %s: %s" % (url, ex))
 
@@ -92,23 +99,19 @@ class ItemView:
         else:
             fl.append(None, f)
         straw.main.poll([f])
-        return
 
     def display_item(self, item):
         self._item = item
         self._document.clear()
         self._document.open_stream("text/html")
         html = (
-            """<html><head>
-            <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
-            <style type="text/css">
-            q { font-style: italic;}
-            blockquote { display: block; font-style: italic; }
-            .stitle {background-color:transparent;font-size:small;font-weight:bold;}
-            .sdate {font-size:small}
-            .content {padding-left:10px;padding-right:40px;margin-top:10px;padding-bottom:10px;}
-            </style>
-            <title>title</title></head><body>""" +
+            """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
+               <html>
+               <head><title>title</title>
+            <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
+               <link rel="stylesheet" type="text/css" href="file://""" +
+               os.path.join(utils.find_lib_dir(), "straw.css") + """" />
+               </head><body>""" +
                 self.htmlify_item(item) +
                 "</body></html>")
         html = html.encode('utf-8')
@@ -132,54 +135,108 @@ class ItemView:
         return
 
     def htmlify_empty_feed(self, feed):
-        return ("<html><head><title>title</title></head><body><p>" +
+        return ("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
+               <html>
+               <head><title>title</title>
+               <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
+               <link rel="stylesheet" type="text/css" href="file://""" +
+               os.path.join(utils.find_lib_dir(), "straw.css") + """" />
+               </head><body><p class=\"emptyfeed\">""" +
                    _("No data yet, need to poll first.") +
                 "</p></body></html>")
 
     def htmlify_item(self, item):
+        feed = straw.main.get_visible_feed()
         ret = []
+        ret.append('<table class="itemhead" cellspacing="0" cellpadding="0">')
         if item.title is not None:
-            ret.append('<div class="stitle">%s</div>' % item.title)
+            ret.append(''.join(('<tr><td class="headleft">', _('Title:'),
+                                '</td><td class="headright">')))
+            if (item.link is not None):
+                ret.append('<a href="%s">%s</a>' % 
+                           (item.link,item.title))
+            else:
+                ret.append(item.title)
+            ret.append('</td></tr>')
+            
         if item.pub_date is not None:
-            timestr = straw.utils.format_date(item.pub_date,
-                                              self._date_format, self._encoding)
-            ret.append('<div class="sdate">%s</div>' % timestr)
-        ret.append('<div class="content">')
+            timestr = straw.utils.format_date(
+                item.pub_date, utils.get_date_format(), self._encoding)
+            ret.append(''.join(('<tr><td class="headleft">', _('Date:'),
+                                '</td><td class="headright">',
+                                str(timestr), '</td></tr>')))
+
+        feedtitle = straw.utils.convert_entities(feed.channel_title)
+        if len(feedtitle) == 0:
+            feedtitle = feed.title
+        if len(feedtitle) == 0:
+            feedtitle = feed.channel_link
+        feedtitle = feedtitle.strip()
+        feedlink = feed.channel_link.strip()
+
+        ret.append(''.join(('<tr><td class="headleft">', _('Feed:'),
+                            '</td><td class="headright">')))
+        islink = len(feedlink) > 0
+        if islink:
+            ret.append('<a href="%s">' % feedlink)
+        ret.append(feedtitle)
+        if islink:
+            ret.append('</a>')
+        ret.append('</td></tr></table>')
+
         if item.description is not None:
-            ret.append('%s ' % item.description)
+	    descr = self.filter_description(item.description)
+            ret.append('<div class="description">%s</div>' % descr)
+
+        if item.publication_name is not None:
+            ret.append('<div class="description">')
+            ret.append('<b>%s:</b> %s<br/>' % (_("Publication"),
+                                               item.publication_name))
+            if item.publication_volume is not None:
+                ret.append('<b>%s:</b> %s ' % (_("Volume"),
+                                                    item.publication_volume))
+                if item.publication_number is not None:
+                    ret.append('( %s )<br />' % item.publication_number)
+            if item.publication_section is not None:
+                ret.append('<b>%s:</b> %s<br />' % (_("Section"),
+                                                    item.publication_section))
+            if item.publication_starting_page is not None:
+                ret.append('<b>%s:</b> %s' % (_("Starting Page"),
+                                              item.publication_starting_page))
+            ret.append('</div>')
+
+        # freshmeat fields
         if item.fm_license != '' and item.fm_license is not None:
-            ret.append('<p><b>%s:</b> %s</p>' % (_("Software license"), item.fm_license))
+            ret.append('<div class="description"><p><b>%s:</b> %s</p>' % (_("Software license"), item.fm_license))
         if item.fm_changes != '' and item.fm_changes is not None:
             ret.append('<p><b>%s:</b> %s</p>' % (_("Changes"), item.fm_changes))
-        if item.license_urls is not None:
-            ret.append("<p>")
-            for l in item.license_urls:
-                ret.append('<a href="%s">%s</a> ' % (l, _("License")))
-            ret.append('</p>')
-        if item.publication_name is not None:
-            ret.append('<p>')
-            ret.append('<b>%s:</b> %s ' % (_("Publication"),
-                                           item.publication_name))
-            if item.publication_volume is not None:
-                ret.append('%s' % item.publication_volume)
-                if item.publication_number is not None:
-                    ret.append('(%s)' % item.publication_number)
-            if item.publication_section is not None:
-                ret.append(' %s' % item.publication_section)
-            if item.publication_starting_page is not None:
-                ret.append(', %s %s' % (_("Page"),
-                                        item.publication_starting_page))
-            ret.append('</p>')
-        ret.append('</div>')
+        # empty paragraph to make sure that we get space here
+        ret.append('<p></p>')
+        # Additional information
+        dcret = []
         if item.creator is not None:
-            ret.append('<p>%s: %s</p>' % (_("Posted by"), item.creator))
-        if (item.link is not None):
-            ret.append('<a href="%s">%s &gt;&gt;</a><br />' %
-                       (item.link,_("Complete story")))
-        if (item.guid is not None) and item.guidislink:
-            ret.append('<a href="%s">%s &gt;&gt;</a><br />' %
-                       (item.guid,_("Permalink")))
+            dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><span>%s</span></td></tr>' % (_("Posted by"), item.creator))
+        if item.contributors is not None and len(item.contributors):
+            for c in item.contributors:
+                dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><span>%s</span></td></tr>' \
+                             % (_("Contributor:"), c.name))
+        if item.guid is not None and item.guid != "" and item.guidislink:
+            dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><a class="dclink" href="%s"><span>%s</span></a></td></tr>' % (_("Permalink"), item.guid, item.guid))
+        # check for not guidislink for the case where there is guid but
+        # isPermalink="false" and yet link is the same as guid (link is
+        # always assumed to be a valid link)
+        if item.link != "" and item.link is not None and (item.link != item.guid or not item.guidislink):
+             dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><a class="dclink" href="%s"><span>%s</span></a></td></tr>' %
+                        (_("Complete story"), item.link, item.link))
+        if item.license_urls is not None:
+            for l in item.license_urls:
+                dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><a class="dclink" href="%s"><span>%s</span></a></td></tr>' % (_("License"), l, l))
 
+        if dcret != []:
+            ret.append('<div class="dcinfo">%s<table class="dc">' % _("Additional information"))
+            ret.append("".join(dcret))
+            ret.append('</table>')
+        ret.append('</div>')
         return "".join(ret)
 
     def scroll_down(self):
@@ -204,6 +261,15 @@ class ItemView:
         self._href_widget.hide()
         self._item_source_vbox.hide()
 
+    def filter_description(self, descr):
+        matchtags = re.compile("<[^>]*>")
+        if re.search(matchtags, descr) != None:
+            # there are tags. return.
+    	    return descr
+        else:
+            # replace \n with <br/>
+            return descr.replace('\n', '<br/>')
+
 class ItemList:
     COLUMN_TITLE = 0
     COLUMN_STICKY = 1
@@ -226,6 +292,7 @@ class ItemList:
         self._item_popup = self._item_factory.get_widget('<item_list_popup>')
         self._widget.connect("popup-menu", self.on_popup_menu)
         self._widget.connect("button_press_event", self.on_button_press_event)
+        self._widget.connect("row_activated", self.on_row_activated)
 
         widget.set_model(model)
         widget.set_rules_hint(False)
@@ -274,10 +341,6 @@ class ItemList:
         count = 0
         visible_iter = None
         for item in feed.items:
-            if not item.title_converted:
-                item.title = straw.utils.convert_title(item.title, item.description)
-                item.title_converted = True
-
             if item.seen:
                 bold = pango.WEIGHT_NORMAL
             else:
@@ -387,6 +450,14 @@ class ItemList:
     def on_menu_mark_as_unread_activate(self, *args):
         straw.main.get_visible_item().seen = False
 
+    def on_row_activated(self, treeview, path, column):
+        model = self._widget.get_model()
+        try:
+            iter = model.get_iter(path)
+        except ValueError:
+            return
+        gnome.url_show(model.get_value(iter,self.COLUMN_ITEM).link)
+
 class OfflineToggle:
     def __init__(self, widget):
         self._widget = widget
@@ -428,113 +499,29 @@ class Toolbar:
     def __init__(self,widget):
         widget_tree = glade.get_widget_tree(widget)
         self._widget = widget
-        self._poll = widget_tree.get_widget('toolbar_poll_button')
-        self._new = widget_tree.get_widget('toolbar_add_button')
+        self._refresh_eb = widget_tree.get_widget(
+            'toolbar_refresh_all_eventbox')
+        self._subscribe_eb = widget_tree.get_widget(
+            'toolbar_subscribe_eventbox')
+        self._find_eb = widget_tree.get_widget('toolbar_find_eventbox')
+        self._scroll_or_next_eb = widget_tree.get_widget(
+            'toolbar_scroll_or_next_eventbox')
         self._tooltips = gtk.Tooltips()
+        self._show_tips()
 
-    def show_tips(self):
-        poll = _("Refresh this category")
-        new =  _("Subscribe")
+    def _show_tips(self):
+        refresh = _("Refresh all feeds")
+        subscribe =  _("Subscribe to a new feed")
+        find = _("Find text in articles")
+        scroll_or_next = _("Scroll article or go to next unread article")
 
-        self._tooltips.set_tip(self._poll, poll, poll)
-        self._tooltips.set_tip(self._new, new, new)
+        self._tooltips.set_tip(self._refresh_eb, refresh, refresh)
+        self._tooltips.set_tip(self._subscribe_eb, subscribe, subscribe)
+        self._tooltips.set_tip(self._find_eb, find, find)
+        self._tooltips.set_tip(self._scroll_or_next_eb, scroll_or_next,
+                               scroll_or_next)
         self._tooltips.enable()
 
-class FeedInfoDisplay:
-    def __init__(self, widget):
-        widget_tree = glade.get_widget_tree(widget)
-        self._widget = widget
-        self._title_hbox = widget_tree.get_widget('feed_info_title_hbox')
-        self._link_display = gnome.ui.HRef(url='', text='')
-        self._description_display = widget_tree.get_widget('feed_info_description')
-        self._description_display.set_redraw_on_allocate(True)
-        self._copyright_display = widget_tree.get_widget('feed_info_copyright')
-        self._feed_label = widget_tree.get_widget('feed_item_label')
-        self._feed_refresh = widget_tree.get_widget('feed_refresh_label')
-
-        self._tooltips = gtk.Tooltips()
-        self._title_hbox.pack_start(self._link_display,expand=gtk.FALSE,fill=gtk.FALSE,padding=6)
-
-        self._config = straw.Config.get_instance()
-
-        self._title = None
-        self._description = None
-        self._copyright = None
-
-    def show(self):
-        self.set_visibility(True)
-        self.conditional_show()
-
-    def hide(self):
-        self.set_visibility(False)
-        self.conditional_show()
-
-    def conditional_show(self):
-        if self._title is not None or self._description is not None or self._copyright is not None:        
-            self._widget.show()
-        else:
-            self._widget.hide()
-
-    def set_visibility(self, mode):
-        self._config.feed_info_visibility = mode
-
-    def display_feed(self, feed):
-        title = straw.utils.convert_entities(feed.channel_title)
-        link = feed.channel_link
-        if len(title) == 0:
-            title = feed.title
-        if len(title) == 0:
-            title = feed.channel_url
-        title = title.strip()
-        if len(title) > 0:
-            self._title = title
-            self._link_display.set_text(saxutils.escape(title))
-            self._link_display.set_url(link)
-            self._link_display.show()
-        else:
-            self._title = None
-            self._link_display.hide()
-
-        self._feed_label.set_label("%s _Articles:" % feed.title)
-        next = feed.next_refresh
-        if next is not None:
-            self._feed_refresh.set_label("Next Refresh: %s" % time.strftime('%A, %d %B %H:%M', time.localtime(next)))
-            self._feed_refresh.show()
-        else:
-            self._feed_refresh.hide()
-            
-        description = feed.channel_description.strip()
-        size = len(description)
-        if size and description != title:
-            if straw.utils.is_html(description):
-                description = straw.utils.read_text(description, size)
-            self._description = description
-            self._description_display.set_text(straw.utils.convert_entities(description))
-            self._description_display.show()
-            
-        else:
-            self._description_display.hide()
-
-        copyright = feed.channel_copyright
-        self._copyright_display.set_text(copyright)
-        if len(copyright):
-            self._copyright = copyright
-            self._copyright_display.show()
-        else:
-            self._copyright = None
-            self._copyright_display.hide()
-        self.conditional_show()
-
-    def set_size(self, size):
-        w = max(size - 10, 0)
-        window_height = self._config.main_window_size[1]
-        max_height = window_height / 4
-        for widget in (self._description_display,
-                       self._copyright_display):
-            widget.set_size_request(w, -1)
-            if widget.size_request()[1] > max_height:
-                widget.set_size_request(w, max_height)
-
 class FindResultList:
     COLUMN_TITLE = 0
     COLUMN_FEED = 1
@@ -792,6 +779,7 @@ class FeedListView:
 
         feed_popup_menu_items = (
             (_("/_Refresh"), None, self.on_menu_poll_selected_activate, 0, "<Item>"),
+            (_("/_Stop Refresh"), None, self.on_menu_stop_poll_selected_activate, 0, "<Item>"),
             (_("/_Mark as Read"), None, self.on_menu_mark_all_as_read_activate, 0, "<Item>"),
             (_("/sep"), None, None, 0, "<Separator>"),
             (_("/_Remove"), None, self.remove_selected_feed, 0, "<Item>"),
@@ -861,8 +849,14 @@ class FeedListView:
             return 1
 
     def on_menu_poll_selected_activate(self, *args):
-        feed = [straw.main.get_visible_feed()]
-        straw.main.poll(feed)
+        feed = straw.main.get_visible_feed()
+        straw.main.poll([feed])[0]
+
+    def on_menu_stop_poll_selected_activate(self, *args):
+        model, iter = self._widget.get_selection().get_selected()
+        adapter = model.get_value(iter, self.COLUMN_OBJECT)
+        feed = adapter.get_feed()
+        feed.router.stop_polling()
 
     def on_menu_mark_all_as_read_activate(self, *args):
         straw.main.mark_as_read()
@@ -902,7 +896,7 @@ class FeedListView:
 
     def display_feed_unselected(self, feed):
         self._first_selection_done = True
-        path = straw.main.get_visible_category().index(feed)
+        path = straw.main.get_visible_category().index_feed(feed)
         self._widget.set_cursor(
             path, self._widget.get_column(0), 0)
         self._widget.get_selection().select_path(path)
@@ -1143,6 +1137,54 @@ class FeedListView:
         path = model.get_path(iter)
         return self.get_parent_with_path(straw.FeedList.get_instance(), path)
 
+class ErrorIndicator:
+    def __init__(self, widget):
+        self._widget = widget
+        self._tooltips = gtk.Tooltips()
+        self._feed_error = None
+        self._category_error = None
+        straw.FeedCategoryList.get_instance().signal_connect(
+            straw.FeedCategoryChangedSignal,
+            self._category_changed)
+
+    def switch_feed(self, feed):
+        self._feed_error = feed.error
+        self._update_view()
+
+    def switch_category(self, category):
+        self._switch_category(category)
+
+    def _switch_category(self, category):
+        if category.subscription is not None:
+            self._category_error = category.subscription.error
+        else:
+            self._category_error = None
+        self._update_view()
+
+    def _category_changed(self, signal):
+        if signal.sender is straw.main.get_visible_category():
+            self._switch_category(signal.sender)
+
+    def _update_view(self):
+        if self._feed_error is None and self._category_error is None:
+            self._widget.hide()
+        else:
+            tooltip = []
+            text = ""
+            if self._category_error is not None:
+                tooltip.append(_("Category error:"))
+                tooltip.append(self._category_error)
+            if self._feed_error is not None:
+                if len(tooltip) > 0:
+                    tooltip.append("\n")
+                tooltip.append(_("Feed error:"))
+                tooltip.append(self._feed_error)
+            if len(tooltip) > 0:
+                text = "\n".join(tooltip)
+            self._tooltips.set_tip(self._widget, text, text)
+            self._tooltips.enable()
+            self._widget.show()
+
 class MainWindow:
     def __init__(self, xml):
 
@@ -1150,8 +1192,6 @@ class MainWindow:
 
         self._item_view = ItemView(
             xml.get_widget('item_view_container'))
-        self._feed_info_view = FeedInfoDisplay(
-            xml.get_widget('feed_info_container'))
         self._find_results_view = FindResultList(
             xml.get_widget('find_results_treeview'))
         self._offline_toggle = OfflineToggle(
@@ -1164,13 +1204,13 @@ class MainWindow:
             xml.get_widget('item_selection_treeview'))
         self._toolbar = Toolbar(
             xml.get_widget('toolbar_default'))
+        self._error_indicator = ErrorIndicator(
+            xml.get_widget('statusbar_error_indicator'))
 
         self._window = xml.get_widget('straw_main')
         self._window.set_default_size(*self._config.main_window_size)
         xml.get_widget('main_main_pane').set_position(self._config.main_pane_position)
         xml.get_widget('main_sub_pane').set_position(self._config.sub_pane_position)
-        self._feed_info_view.set_size(self._config.main_pane_position)
-
         self._mode_view_notebook = xml.get_widget('mode_view_notebook')
         self._feed_selection_vbox = xml.get_widget('feed_selection_vbox')
 
@@ -1184,15 +1224,6 @@ class MainWindow:
 
         self._menu_feed_properties = xml.get_widget('menu_feed_properties')
         self._menu_feed_properties.set_sensitive(False)
-        self._menu_feed_info = glade.get_widget_tree(xml.get_widget('menubar_default')).get_widget('menu_feed_information')
-        if self._config.feed_info_visibility:
-            self._menu_feed_info.set_active(gtk.TRUE)
-            self._feed_info_view.show()
-        else:
-            self._menu_feed_info.set_active(gtk.FALSE)
-            self._feed_info_view.hide()
-
-        self._toolbar.show_tips()
         self._offline_toggle.show()
 
         for key in dir(self.__class__):
@@ -1204,12 +1235,16 @@ class MainWindow:
         self._mmgr.signal_connect(straw.StatusDisplaySignal,
                                   self.display_status_messages)
 
+        self._feed_label = xml.get_widget('feed_item_label')
+        self._feed_refresh = xml.get_widget('feed_refresh_label')
+
         self._feedlist = straw.FeedList.get_instance()
 
     def category_select(self, category, *args):
         straw.main.set_visible_category(category)
         self.display_feeds(
             category, straw.main.get_visible_feed())
+        self._error_indicator.switch_category(category)
 
     def get_find_results_view(self):
         return self._find_results_view
@@ -1233,7 +1268,6 @@ class MainWindow:
         config = straw.Config.get_instance()
         def check_position((position, widget)):
             if position == widget.get_position():
-                self._feed_info_view.set_size(position)
                 config.main_pane_position = position
         pos = widget.get_position()
         if pos != config.main_pane_position:
@@ -1251,13 +1285,16 @@ class MainWindow:
     def on_menu_about_activate(self, menuitem, *args):
         description = _("Straw is a desktop news aggregator for GNOME")
         about = gnome.ui.About(
-            straw.APPNAME, straw.VERSION, "Copyright (c) 2002-2003 Juri Pakaste", description,
+            straw.APPNAME, straw.VERSION, "Copyright (c) 2002-2004 Juri Pakaste", description,
             ["Juri Pakaste <juri@iki.fi>",
              "Jan Alonzo <jmalonzo@unpluggable.com>",
+             "Iain McCoy <iain@mccoy.id.au>",
+             "Lucas Nussbaum <lucas@lucas-nussbaum.net>",
+             "Olivier Crete <tester@tester.ca>",
              "Scott Douglas-Watson <sdouglaswatson@yahoo.co.uk>",
              u"Terje R\xf8sten (distutils)",
-             "Mark Pilgrim (feedparser and rssfinder)",
-             "Icons by Jakub 'jimmac' Steiner and Juri Pakaste"],
+             "Mark Pilgrim (feedparser and feedfinder)",
+             "Jakub 'jimmac' Steiner and Juri Pakaste (icons)"],
             [],
             "Juri Pakaste <juri@iki.fi>\n"
             "Martin Steldinger <tribble@hanfplantage.de>\n"
@@ -1280,8 +1317,8 @@ class MainWindow:
         straw.main.show_feed_properties(straw.main.get_visible_feed())
 
     def on_menu_refresh_all_activate(self, *args):
-        feeds = self._feedlist.flatten_list()
-        straw.main.poll(feeds)
+        straw.main.poll_categories(
+            straw.FeedCategoryList.get_instance().all_categories)
 
     def on_menu_refresh_selected_activate(self, *args):
         feed = straw.main.get_visible_feed()
@@ -1289,14 +1326,21 @@ class MainWindow:
 
     def on_menu_refresh_category_activate(self, *args):
         category = straw.main.get_visible_category()
-        straw.main.poll(category)
+        straw.main.poll_categories([category])
 
-    def on_toolbar_poll_button_clicked(self, *args):
-        category = straw.main.get_visible_category()
-        straw.main.poll(category)
+    def on_toolbar_refresh_all_button_clicked(self, *args):
+        straw.main.poll_categories(
+            straw.FeedCategoryList.get_instance().all_categories)
 
-    def on_toolbar_add_button_clicked(self, *args):
-        straw.main.show_subscribe_dialog()
+    def on_toolbar_subscribe_button_clicked(self, *args):
+        SubscribeDialog.show()
+
+    def on_toolbar_scroll_or_next_button_clicked(self, *args):
+        if not self._item_view.scroll_down():
+            straw.main.display_next_unread_item()
+
+    def on_toolbar_find_button_clicked(self, *args):
+        self.show_find_mode()
 
     def on_menu_next_activate(self, *args):
         straw.main.display_next_item()
@@ -1305,9 +1349,6 @@ class MainWindow:
         straw.main.display_previous_item()
 
     def on_menu_scroll_next_activate(self, *args):
-        if self._category_selector.is_focused():
-            self._category_selector.popup()
-            return
         if not self._item_view.scroll_down():
             straw.main.display_next_unread_item()
 
@@ -1323,12 +1364,6 @@ class MainWindow:
         straw.main.display_next_unread_feed()
         return
 
-    def on_menu_feed_information_activate(self, *args):
-        if self._menu_feed_info.get_active():
-            self._feed_info_view.show()
-        else:
-            self._feed_info_view.hide()
-
     def on_menu_find_activate(self, *args):
         self.show_find_mode()
 
@@ -1336,7 +1371,7 @@ class MainWindow:
         self._feed_list_view.remove_selected_feed()
 
     def on_menu_add_activate(self, *args):
-        straw.main.show_subscribe_dialog()
+        SubscribeDialog.show()
 
     def on_menu_import_subscriptions_activate(self, *args):
         straw.file_selector.subscriptions_import()
@@ -1383,13 +1418,18 @@ class MainWindow:
     def show_find_results(self, items):
         self._find_results_view.render_results(items)
 
+
     def display_feed(self, feed, select_first = 1):
-        if feed.error is None:
-            self._mmgr.post_message('')
+        self._error_indicator.switch_feed(feed)
+        self._feed_label.set_label("%s _Articles:" % feed.title)
+        next = feed.next_refresh
+        if next is not None:
+            self._feed_refresh.set_label(_("Next Refresh: %s") % time.strftime(utils.get_date_format(), time.localtime(next)))
+            self._feed_refresh.show()
         else:
-            self._mmgr.post_message("Error while processing feed: %s" % feed.error)
+            self._feed_refresh.hide()
+
         self._item_list_view.display_feed(feed, select_first)
-        self._feed_info_view.display_feed(feed)
         self._menu_feed_properties.set_sensitive(True)
 
     def display_feed_unselected(self, feed):
@@ -1397,7 +1437,7 @@ class MainWindow:
         self._menu_feed_properties.set_sensitive(True)
 
     def display_empty_feed(self, feed):
-        if feed.error is None:
+        if feed.error != "":
             self._mmgr.post_message('')
         else:
             self._mmgr.post_message("Error while processing feed: %s" % feed.error)
@@ -1451,7 +1491,7 @@ class MainWindow:
 
     def display_feeds(self, category, visible_feed):
         self._category_selector.set_selected(category)
-        self._feed_list_view.display_feeds(category, visible_feed)
+        self._feed_list_view.display_feeds(category.feeds, visible_feed)
         self.update_total_unread_indicator()
 
     def display_categories(self):
@@ -1462,7 +1502,7 @@ class MainWindow:
 
     def on_straw_main_drag_data_received(self, w, context, x, y, data, info, time):
         if data and data.format == 8:
-            straw.main.show_subscribe_dialog(data.data.split("\n")[0])
+            SubscribeDialog.show(data.data.split("\n")[0])
             context.finish(gtk.TRUE, gtk.FALSE, time)
         else:
             context.finish(gtk.FALSE, gtk.FALSE, time)
@@ -1473,3 +1513,6 @@ class MainWindow:
     def enable_feed_selection(self):
         self._feed_list_view.enable_feed_selection()
 
+    def present(self):
+        self._window.present()
+
