@@ -1,11 +1,9 @@
-cvs_id = "$Id: OPMLExport.py,v 1.2 2004/04/16 20:26:07 jpakaste Exp $"
-
 import straw
 import gnome.vfs
 
-def export(list, fname):
+def export(title, list, fname):
     opml = straw.OPML.OPML()
-    opml['title'] = list.title
+    opml['title'] = title
     for feed in list:
         o = straw.OPML.Outline()
         o['text'] = feed.title.encode('utf-8')
